# -*- coding: utf-8 -*-
"""
database orm sql package.
"""

from pyrin.packaging.context import Package


class DatabaseORMSQLPackage(Package):
    """
    database orm sql package class.
    """

    NAME = __name__
