# -*- coding: utf-8 -*-
"""
locale babel package.
"""

from pyrin.packaging.context import Package


class LocaleBabelPackage(Package):
    """
    locale babel package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'globalization.locale.babel.component'
