# -*- coding: utf-8 -*-
"""
token handlers package.
"""

from pyrin.packaging.context import Package


class TokenHandlersPackage(Package):
    """
    token handlers package class.
    """

    NAME = __name__
    DEPENDS = []
