# -*- coding: utf-8 -*-
"""
database orm sql extractor package.
"""

from pyrin.packaging.context import Package


class DatabaseORMSQLExtractorPackage(Package):
    """
    database orm sql extractor package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'database.orm.sql.extractor.component'
